function out=phaseRetrievalTVOverAccer(b0,b1,b2,s,t,lambda,r1,r2,itmax,I,Omega,init,verbose,r3,flag)
%TV Lasso oversampling  with additional v=u
%% Huibin Chang 2014.1
if nargin<11
   Omega=ones(size(b0));
end

if ~exist('flag','var')
    flag=1;
end

%%

[m,n]=size(b0);
p=zeros(m,2*n);
d0=zeros(m,n);
d1=d0;
d2=d0;
q=p;

z0=zeros(m,n);
z1=z0;
z2=z0;
%M=length(find(Omega>0));
%% prepare DFT
[m,n]=size(I);
x=exp(1i*2*pi*s*[0:m-1]/m);
y=exp(1i*2*pi*t*[0:n-1]/n);
[x,y]=meshgrid(x,y);
D=x.*y;
%%

if ~exist('uinit','var')
z0(Omega)=b0(Omega).*exp(1i*2*pi*init(:,1));
z1(Omega)=b1(Omega).*exp(1i*2*pi*init(:,2));
z2(Omega)=b2(Omega).*exp(1i*2*pi*init(:,3));
else   
z0=Myfftn(uinit);
z1=Myfftn(uinit+    D.*uinit);
z2=Myfftn(uinit- 1i*D.*uinit);
%itmax=100;
end
%% prepare A of Laplacian A+ I
[r,c]=size(I);
n = r*c;

%Build A matrix


A =spdiags(ones(n,1),0,n,n);
A=4*A-circshift(A,[0,-1])-circshift(A,[0,1])-circshift(A,[0,r])-circshift(A,[0,-r]);
A=sparse(A);




A=A+2*r1/r2*(2.5*speye(size(A))+spdiags(real(D(:)),0,size(A,1),size(A,2))...
                               +spdiags(imag(D(:)),0,size(A,1),size(A,2))     );


A=A+r3*speye(size(A))/r2;




[~,n]=size(I);

%%
%
%start iteration
ETotal=zeros(itmax,1);
ETV=ETotal;
EFit=ETotal;
ER=ETotal;
uOld=zeros(size(I));
w=zeros(size(I));
% v=w;
v=real(Myifftn(z0));
   for counter=1:itmax
       
       
    
       
       
       z0hat=Myifftn(z0+d0/r1);
       z1hat=Myifftn(z1+d1/r1);
       z2hat=Myifftn(z2+d2/r1);
       
       % for u sub
       %vtmp=v-d/r1;
       ptmp=p+q/r2;
       g=r1/r2*(real(z0hat+z1hat+z2hat)+real(D.*conj(z1hat))+imag(D.*conj(z2hat)))...
            -(dxb(ptmp(:,1:n)+dyb(ptmp(:,n+1:2*n))))+1/r2*(r3*v-w);
%        u=poissonFFT(g/r2,r1/r2); 
       if counter==1
          u=cgs(A,g(:));
       else
       [u,~]=cgs(A,g(:),1e-7,1,[],[],u(:));
       %[u]=cgs(A,g(:),1e-4,1,[],[],u(:));
       end
       

       u=reshape(u,size(I,1),size(I,2));
       
    
       v=min(max(0,u+w/r3),255);
       
       
       
       
       if verbose
       figure(1);imshow(u,[]);colorbar;title(num2str(counter));pause(.001)
       end
       
       
       
       
       % for v-sub
       
       z0=subZSolver(b0,Myfftn(u)-d0/r1,lambda,r1,Omega);
       z1=subZSolver(b1,Myfftn(u+   D.*u)-d1/r1,lambda,r1,Omega);
       z2=subZSolver(b2,Myfftn(u-1i*D.*u)-d2/r1,lambda,r1,Omega);
           
         
      
        %for p
       p=softThresh([dxf(u) dyf(u)]-q/r2,1/r2);
       
      

       d0=d0+r1*(z0-Myfftn(u));
       d1=d1+r1*(z1-Myfftn(u+   D.*u));
       d2=d2+r1*(z2-Myfftn(u-1i*D.*u)); 
       q =q +r2*(p-[dxf(u) dyf(u)]); 
       w =w +r3*(u-v);
       
         %r1=r1*1.05;
         %r2=r2*1.05;
       %err=norm(uOld-u,'fro')
       %uOld=u;
       
       if flag==1
       ER(counter,1)=norm(uOld-u,'fro')/norm(u,'fro');
       
       ETV(counter,1)=TV(u);
       EFit(counter,1)=L2Term(u,D,b0,b1,b2,Omega);
       ETotal(counter,1)=ETV(counter,1)+lambda*EFit(counter,1)/2;
       end
       
       uOld=u;

   end
   
   out.u=u;
   out.v=v;
   out.ETV=ETV;
   out.EFit=EFit;
   out.ETotal=ETotal;
   out.ER=ER;


end







function p=softThresh(p0,beta)
[~,n]=size(p0);n=n/2;
Z1=p0(:,1:n);
Z2=p0(:,n+1:2*n);
    V = abs(Z1).^2 + abs(Z2).^2;
    V = sqrt(V);
    
    V(V==0) = 1;
    
    V = max(V - beta, 0)./V;
    
    
    Y1 = Z1.*V;
    Y2 = Z2.*V;
    
    %Y1=Y1.*(1-Flag)+max(0,-beta).*Flag*sqrt(.5);
    %Y2=Y2.*(1-Flag)+max(0,-beta).*Flag*sqrt(.5);
 p=[Y1 Y2];   
end

function u=Myfftn(u0)
u=fftn(u0)/sqrt(numel(u0));
end

function u=Myifftn(u0)
u=ifftn(u0)*sqrt(numel(u0));
end
function z=subZSolver(b,z0,lambda,r,Omega)
        
       z=(lambda*b+r*abs(z0))./(lambda+r).*sign(z0);
       z=z0.*(1-Omega)+z.*Omega;
end

function f=TV(u)

u=sqrt(dxf(u).^2+dyf(u).^2);
f=sum(sum(u));
end
function f=L2Term(u,D,b0,b1,b2,Omega)

 u=(abs(Myfftn(u))-b0).^2+ (abs(Myfftn(u+   D.*u))-b1).^2+(abs( Myfftn(u-1i*D.*u))-b2).^2;
 u=u.*Omega;
 f=sum(u(:));
end

